# Generated by Django 3.1.2 on 2021-03-29 09:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0005_auto_20210326_1628'),
    ]

    operations = [
        migrations.AddField(
            model_name='reductionrun',
            name='reduction_host',
            field=models.TextField(blank=True, default='', verbose_name='Reduction hostname'),
        ),
    ]
