# Generated by Django 3.1.2 on 2021-03-26 16:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0004_instrument_is_flat_output'),
    ]

    operations = [
        migrations.AlterField(
            model_name='experiment',
            name='reference_number',
            field=models.IntegerField(unique=True),
        ),
    ]
