# Generated by Django 3.1.2 on 2021-03-10 16:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0002_remove_reductionrun_cancel'),
    ]

    operations = [
        migrations.RenameField(
            model_name='reductionrun',
            old_name='run_name',
            new_name='run_description',
        ),
    ]
